#ifndef MSP_GL_AMBIENTOCCLUSION_H_
#define MSP_GL_AMBIENTOCCLUSION_H_

#include "framebuffer.h"
#include "mesh.h"
#include "postprocessor.h"
#include "program.h"
#include "programdata.h"
#include "texture2d.h"
#include "texturing.h"

namespace Msp {
namespace GL {

/**
Implements screen-space ambient occlusion.

http://en.wikipedia.org/wiki/Screen_Space_Ambient_Occlusion
*/
class AmbientOcclusion: public PostProcessor
{
private:
	Texture2D occlusion;
	Texture2D rotate_lookup;
	Framebuffer fbo;
	Texturing occlude_texturing;
	Program occlude_shader;
	ProgramData occlude_shdata;
	Texturing combine_texturing;
	Program combine_shader;
	ProgramData combine_shdata;
	const Mesh &quad;

public:
	AmbientOcclusion(unsigned, unsigned, float);

	void set_depth_ratio(float);
	void set_darkness(float);

	virtual void render(Renderer &, const Texture2D &, const Texture2D &);
};

} // namespace GL
} // namespace Msp

#endif
