#ifndef MSP_GL_EFFECT_H_
#define MSP_GL_EFFECT_H_

#include <set>
#include "renderable.h"

namespace Msp {
namespace GL {

/**
Effects are used to wrap other renderables and give them additional visual
properties.  An Effect's render method should set up the necessary state, call
the wrapped Renderable's render method, and clean up after itself.
*/
class Effect: public Renderable
{
protected:
	const Renderable &renderable;
	std::set<Tag> enabled_passes;

	Effect(const Renderable &);
public:
	virtual ~Effect() { }

	void enable_for_pass(const Tag &);
	void disable_for_pass(const Tag &);

	virtual const Matrix *get_matrix() const { return renderable.get_matrix(); }

	virtual void setup_frame() const { renderable.setup_frame(); }
	virtual void finish_frame() const { renderable.finish_frame(); }
};

} // namespace GL
} // namespace Msp

#endif
